/*
 * block_io.h
 *
 * Copyright 2004 Nigel Cunningham <ncunningham@linuxmail.org>
 *
 * Distributed under GPLv2.
 *
 * This file contains declarations for functions exported from
 * block_io.c, which contains low level io functions.
 */

/* 
 * The maximum amount of I/O we submit at once.
 */
#define MAX_READAHEAD 1024

/* Forward Declarations */

/*
 * submit_params
 *
 * The structure we use for tracking submitted I/O.
 */
struct submit_params {
	swp_entry_t swap_address;
	struct page * page;
	struct block_device * dev;
	long blocks[PAGE_SIZE/512];
	int blocks_used;
	int readahead_index;
	struct submit_params * next;
};


/* 
 * Our exported interface so the swapwriter and filewriter don't
 * need these functions duplicated.
 */
struct suspend_bio_ops {
	int (*set_block_size) (struct block_device * bdev, int size);
	int (*get_block_size) (struct block_device * bdev);
	int (*submit_io) (int rw, 
		struct submit_params * submit_info, int syncio);
	int (*bdev_page_io) (int rw, struct block_device * bdev, long pos,
			struct page * page);
	void (*wait_on_readahead) (int readahead_index);
	void (*check_io_stats) (void);
	void (*reset_io_stats) (void);
	void (*finish_all_io) (void);
	int (*prepare_readahead) (int index);
	void (*cleanup_readahead) (int index);
	struct page ** readahead_pages;
	int (*readahead_ready) (int readahead_index);
};

extern struct suspend_bio_ops suspend_bio_ops;
