/*
 * kernel/power/suspend2_core/pagedir.h
 *
 * Copyright (C) 2004 Nigel Cunningham <ncunningham@linuxmail.org>
 *
 * This file is released under the GPLv2.
 *
 * Declarations for routines for handling pagesets.
 * Note that pbes aren't actually stored as such. The data is stored in
 * bitmaps.
 */

/* Page Backup Entry.
 *
 * This is an abstraction which contains the data for one
 * page of the image. (The data is really stored in extents).
 */

#ifndef SUSPEND2_CORE_PAGEDIR_H
#define SUSPEND2_CORE_PAGEDIR_H

/* Pagedir
 *
 * Contains the metadata for a set of pages saved in the image.
 */
struct pagedir {
	int pageset_size;
	int lastpageset_size;
};

#define pageset1_size (pagedir1.pageset_size)
#define pageset2_size (pagedir2.pageset_size)

/* Pagedir1 is the metadata for pageset 1 pages. Likewise for pageset 2. */
extern struct pagedir pagedir1, pagedir2;

extern void copy_pageset1(void);
#ifdef CONFIG_DEBUG_PAGEALLOC
extern void suspend_map_atomic_copy_pages(void);
extern void suspend_unmap_atomic_copy_pages(void);
#endif
extern void free_pagedir_data(void);
extern int PageInPagedir(struct pagedir * p, struct page * page);
extern int allocate_extra_pagedir_memory(struct pagedir * p,
		int pageset_size, int alloc_from);
extern void mark_pages_for_pageset2(void);
extern unsigned long get_nonconflicting_page(void);
extern void release_conflicting_pages(void);
extern void relocate_page_if_required(void ** page_pointer_addr);
extern int get_pageset1_load_addresses(void);
extern void set_chain_names(struct pagedir * p);
#endif
